﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;

namespace TetrisGUI
{
    public partial class TetrisMainWnd : Form
    {
        [DllImport("user32.dll")]
        private static extern short GetAsyncKeyState(int vKey);
        private enum VirtualKeyCode : int {
            VK_LEFT  = 0x25,
            VK_UP    = 0x26,
            VK_RIGHT = 0x27,
            VK_DOWN  = 0x28
        }

        protected const int TetrisBlockNumX = 10; // 横方向ブロック数
        protected const int TetrisBlockNumY = 20; // 縦方向ブロック数
        protected const int BlockSize = 36;       // ブロックのサイズ

        // テトリスガイドラインの色使い
        enum TetrisBlockState {
	        None,     // 無し
	        Cyan,     // 水色 (棒)
	        Yellow,   // 黄 (正方形)
	        Green,    // 緑 (S字)
	        Red,      // 赤 (Z字)
	        Blue,     // 青 (J字)
	        Orange,   // オレンジ (L字)
	        Magenta   // 紫 (T字)
        };

        protected TetrisMgrIf tetrisMgr;
        protected ImageList blockImageList; // Blockの絵のイメージリスト

        public TetrisMainWnd()
        {
            InitializeComponent();

            tetrisMgr = new TetrisMgrIf();

            blockImageList = new ImageList();
            blockImageList.ImageSize = new Size(BlockSize, BlockSize);
            blockImageList.Images.AddStrip(TetrisGUI.Properties.Resources.BLOCKS); // 埋め込んだリソースから読み込み

            this.ClientSize = new Size(TetrisBlockNumX * BlockSize, TetrisBlockNumY * BlockSize); // ウインドウサイズ調整

            // ダブルバッファリングを有効にする
            SetStyle(ControlStyles.DoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);

            // タイマーのセットアップ
            timerFall.Interval = (int)tetrisMgr.GetFallTimerInterval();
            timerBtnChk.Enabled = true;
            timerFall.Enabled = true;

            Invalidate();
        }

        // 描画
        protected override void OnPaint(PaintEventArgs e)
        {
            for (int i = 0; i < TetrisBlockNumX; i++)
            {
                for (int j = 0; j < TetrisBlockNumY; j++)
                {
                    blockImageList.Draw(e.Graphics, i * BlockSize, j * BlockSize, (int)tetrisMgr.GetBlockState((uint)i, (uint)j));
                }
            }

            base.OnPaint(e);
        }

        // タイマーハンドラ
        private void timerBtnChk_Tick(object sender, EventArgs e)
        {
            if ((GetAsyncKeyState((int)VirtualKeyCode.VK_LEFT) & 0x8000) == 0x8000)
            {
                tetrisMgr.OnLeft();
                Invalidate();
            }
            else if ((GetAsyncKeyState((int)VirtualKeyCode.VK_RIGHT) & 0x8000) == 0x8000)
            {
                tetrisMgr.OnRight();
                Invalidate();
            }
            else if ((GetAsyncKeyState((int)VirtualKeyCode.VK_UP) & 0x8000) == 0x8000)
            {
                tetrisMgr.OnLRotate();
                Invalidate();
            }
            else if ((GetAsyncKeyState((int)VirtualKeyCode.VK_DOWN) & 0x8000) == 0x8000)
            {
                tetrisMgr.OnDown();
                Invalidate();
            }
            else if ((GetAsyncKeyState((int)'Z') & 0x8000) == 0x8000) // VK_LEFT
            {
                tetrisMgr.OnLRotate();
                Invalidate();
            }
            else if ((GetAsyncKeyState((int)'X') & 0x8000) == 0x8000) // VK_LEFT
            {
                tetrisMgr.OnRRotate();
                Invalidate();
            }
            TimerCommonCheck();
        }

        private void timerFall_Tick(object sender, EventArgs e)
        {
            tetrisMgr.OnFall();
            Invalidate();
            TimerCommonCheck();
        }

        private void TimerCommonCheck()
        {
            if (tetrisMgr.IsGameOver() != 0)
            {
                // ゲームオーバーになった
                // まずタイマー停止
                timerBtnChk.Enabled = false;
                timerFall.Enabled = false;

                if (MessageBox.Show("Game Over... Retry?", "Game Over",
                    MessageBoxButtons.OKCancel, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1) == DialogResult.OK)
                {
                    // 再スタートが要求された
                    tetrisMgr.OnInit();
                    tetrisMgr.OnStart();

                    timerFall.Interval = (int)tetrisMgr.GetFallTimerInterval();
                    timerBtnChk.Enabled = true;
                    timerFall.Enabled = true;

                    Invalidate();
                }
                else
                {
                    // 終了
                    Close();
                }
            }
            else if (timerFall.Interval != (int)tetrisMgr.GetFallTimerInterval())
            {
                // 落ちる速度が変わった
                timerFall.Enabled = false;
                timerFall.Interval = (int)tetrisMgr.GetFallTimerInterval();
                timerFall.Enabled = true;
            }
        }
    }
}
